///////////////////////////////////////////////////////////////////
// TravellogManager.cpp
// ߂iޗǗNX
///////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "TravellogManager.h"
#include "PatriotView.h"
#include "resource.h"
#include "patriotGlobal2.h"
#include "patriot.h"

#include <tlogstg.h>

///////////////////////////////////////////////////////////////////
/// CTravellogManager
///////////////////////////////////////////////////////////////////

const int TRAVELLOG_MAX_COUNT = 9;

/// RXgN^
CTravelLogManager::CTravelLogManager(CPatriotView* pView)
{
	m_pView 		  = pView;
	m_state 		  = STATE_NONE;

	m_pBackEntries	  = NULL;
	m_pForwardEntries = NULL;
}

/// fXgN^
CTravelLogManager::~CTravelLogManager()
{
	if (m_pBackEntries || m_pForwardEntries) {
		delete[] m_pBackEntries;
		delete[] m_pForwardEntries;
	}
}

/// ǂݍ
void CTravelLogManager::Load()
{
	if (m_state != STATE_NONE) {
		return;
	}

	Clear();

	if (m_pView->m_bBackButton) {
		LoadTravelLog(TLEF_RELATIVE_BACK);
	}
	if (m_pView->m_bForwardButton) {
		LoadTravelLog(TLEF_RELATIVE_FORE);
	}
	m_state = STATE_LOADED;
}

/// 𕜌
void CTravelLogManager::Restore()
{
	int nBackCount				= RestoreTravelLog(TLEF_RELATIVE_BACK);
	m_pView->m_bBackButton		= (nBackCount > 0);

	int nForwardCount			= RestoreTravelLog(TLEF_RELATIVE_FORE);
	m_pView->m_bForwardButton	= (nForwardCount > 0);

	m_state = STATE_LOADED;
}

/// j
void CTravelLogManager::Clear()
{
	m_pBackEntries	  = EmptyEntryList(m_pBackEntries);
	m_pForwardEntries = EmptyEntryList(m_pForwardEntries);

	m_state = STATE_NONE;
}

/// Xg
CTravelLogEntry* CTravelLogManager::EmptyEntryList(CTravelLogEntry* pEntries)
{
	if (pEntries) {
		if (m_state != STATE_NONE) {
			for (int i = 0; i < TRAVELLOG_MAX_COUNT; i++) {
				pEntries[i].Empty();
			}
		}
	} else {
		pEntries = new CTravelLogEntry[TRAVELLOG_MAX_COUNT];
	}
	return pEntries;
}

/// XgRs[
CTravelLogEntry* CTravelLogManager::CopyEntryList(CTravelLogEntry* pEntriesTo, CTravelLogEntry* pEntriesFrom)
{
	if (pEntriesTo && pEntriesFrom) {
		for (int i = 0; i < TRAVELLOG_MAX_COUNT; i++) {
			pEntriesTo[i] = pEntriesFrom[i];
		}
	}
	return pEntriesTo;
}

/// ߂Eiޗǂݍ
int CTravelLogManager::LoadTravelLog(int direction)
{
	try {
		CTravelLogEntry* pEntries = (direction == TLEF_RELATIVE_BACK) ? m_pBackEntries : m_pForwardEntries;
		int 			 nIndex   = 0;

		HRESULT 		 hr 	  = S_OK;
		CComPtr<IDispatch>			pDocDisp= m_pView->GetHtmlDocument();
		CComQIPtr<IServiceProvider> pISP	= pDocDisp;
		if (pISP == NULL) {
			return 0;
		}

		CComPtr<ITravelLogStg> pTLStg;
		hr = pISP->QueryService(SID_STravelLogCursor, IID_ITravelLogStg, (void**)&pTLStg);
		if (FAILED(hr)) {
			return 0;
		}

		CComPtr<IEnumTravelLogEntry> pTLEnum;
		hr = pTLStg->EnumEntries(direction, &pTLEnum);

		if (SUCCEEDED(hr) && pTLEnum) {
			ITravelLogEntry* pTLEntry = NULL;
			hr = pTLEnum->Next(1, &pTLEntry, NULL);
			while (hr != S_FALSE) {
				LPOLESTR	szTitle = NULL;
				hr = pTLEntry->GetTitle(&szTitle);
				if (SUCCEEDED(hr) && szTitle){
					pEntries[nIndex].title = szTitle;
					::CoTaskMemFree(szTitle);

					LPOLESTR	szURL = NULL;
					hr = pTLEntry->GetURL(&szURL);
					if (SUCCEEDED(hr) && szURL){
						pEntries[nIndex].URL = szURL;
						::CoTaskMemFree(szURL);
						nIndex++;
						if (nIndex == TRAVELLOG_MAX_COUNT) {
							break;
						}
					}
				}

				pTLEntry->Release();
				pTLEntry = NULL;

				if (nIndex >= TRAVELLOG_MAX_COUNT) {
					break;
				}
				hr = pTLEnum->Next(1, &pTLEntry, NULL);
			}
		}
		return nIndex + 1;
	} catch (CException* e) {
		e->ReportError();
	} catch (...) {
		AfxMessageBox(_T("Error:LoadTravelLog"));
	}
	return 0;
}

/// ߂Eiޗ𕜌
int CTravelLogManager::RestoreTravelLog(int direction)
{
	try {
		CTravelLogEntry* pEntries = (direction == TLEF_RELATIVE_BACK) ? m_pBackEntries : m_pForwardEntries;
		if (pEntries[0].URL.IsEmpty()) {
			return 0;
		}

		HRESULT hr = S_OK;
		CComPtr<IDispatch> spDocDisp = m_pView->GetHtmlDocument();
		CComQIPtr<IServiceProvider> spISP = spDocDisp;
		if (spISP == NULL) {
			return 0;
		}

		CComPtr<ITravelLogStg> spTLStg;
		hr = spISP->QueryService(SID_STravelLogCursor, IID_ITravelLogStg, (void**)&spTLStg);
		if (FAILED(hr)) {
			return 0;
		}

		CComPtr<IEnumTravelLogEntry> spTLEnum;
		hr = spTLStg->EnumEntries(direction, &spTLEnum);

		CComPtr<ITravelLogEntry> spTLEntry;
		hr = spTLEnum->Next(1, &spTLEntry, NULL);
		if (FAILED(hr)) {
			return 0;
		}

		USES_CONVERSION;

		int i = 0;
		while (!pEntries[i].URL.IsEmpty()) {
			CComPtr<ITravelLogEntry> spNewTLEntry;
			LPCOLESTR	szTitle 	= T2COLE(pEntries[i].title);
			LPCOLESTR	szUrl		= T2COLE(pEntries[i].URL);
			BOOL		bDirection	= (direction == TLEF_RELATIVE_BACK);
			//spTLStg->CreateEntry(url, title, in_ppEntry, back_as_TRUE_or_forward_as_FALSE, out_ppEntry);
			hr = spTLStg->CreateEntry( szUrl, szTitle, spTLEntry, bDirection, &spNewTLEntry );
			spTLEntry.Release();

			spTLEntry = spNewTLEntry;
			i++;
			if (i == TRAVELLOG_MAX_COUNT) {
				break;
			}
		}
		return i;
	} catch (CException* e) {
		e->ReportError();
	} catch (...) {
		AfxMessageBox(_T("Error:RestoreTravelLog"));
	}
	return 0;
}

/// j[쐬
void CTravelLogManager::CreateMenu(CMenu* pMenu, int direction)
{
	if (m_state == STATE_NONE) {
		Clear();
	}

	CTravelLogEntry* pEntries = (direction == MENU_BACK) ? m_pBackEntries : m_pForwardEntries;
	UINT nMenuIDStart = (direction == MENU_BACK) ? ID_VIEW_BACK_HISTORY : ID_VIEW_FORWARD_HISTORY;

	if (m_state == STATE_NONE) {
		if (direction == MENU_BACK) {
			LoadTravelLog(TLEF_RELATIVE_BACK);
		} else {
			LoadTravelLog(TLEF_RELATIVE_FORE);
		}
	}

	// ̃^Cgj[ɒǉ
	int i = 0;
	while (i < TRAVELLOG_MAX_COUNT && !pEntries[i].title.IsEmpty()) {
		pMenu->ModifyMenu( nMenuIDStart + i, MF_BYCOMMAND | MF_STRING, nMenuIDStart + i, pEntries[i].title);
		i++;
	}
	// Ȃj[폜
	for (int j = i; j < TRAVELLOG_MAX_COUNT; j++) {
		pMenu->RemoveMenu( nMenuIDStart + j, MF_BYCOMMAND );
	}
}

/// Rs[
void CTravelLogManager::Copy(CTravelLogManager* src)
{
	Copy(src->m_pBackEntries, src->m_pForwardEntries);
}

/// Rs[
void CTravelLogManager::Copy(CTravelLogEntry* pBackEntries, CTravelLogEntry* pForwardEntries)
{
	Clear();

	CopyEntryList(m_pBackEntries,	 pBackEntries);
	CopyEntryList(m_pForwardEntries, pForwardEntries);
}

/// Rs[
void CTravelLogManager::CopyTo(CTravelLogEntry* pBackEntries, CTravelLogEntry* pForwardEntries)
{
	CopyEntryList(pBackEntries,    m_pBackEntries);
	CopyEntryList(pForwardEntries, m_pForwardEntries);
}
